package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesInspectionProductDataReportDao;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesInspectionProductDataAmountReportModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.report.ShippingReportModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class MesInspectionProductDataReportDao implements IMesInspectionProductDataReportDao {

    @Autowired
    private EntityManager entityManager;

    /**
     * 检验产品数据总报表-数量
     * @param model
     * @return
     */
    @Override
    public int queryInspectionProductDataAmountReportCount(MesInspectionProductDataAmountReportModel model) {
        StringBuffer hql = new StringBuffer();
        hql.append(" select count(1) ");

        packHqlForQuery(hql, model);

        Query query = entityManager.createQuery(hql.toString(), Long.class);

        packParameterForQuery(query, model);

        return ((Long) query.getSingleResult()).intValue();
    }

    /**
     * 检验产品数据总报表
     * @param model
     * @param pager
     * @return
     */
    @Override
    public List<MesInspectionProductDataAmountReportModel> queryInspectionProductDataAmountReport(MesInspectionProductDataAmountReportModel model, Pager pager) {

        StringBuffer hql = new StringBuffer();
        hql.append("select new ").append(MesInspectionProductDataAmountReportModel.class.getName()).append(" ( ");
        // 添加所有需要查询的字段
        hql.append("mpi.id,");
        hql.append("mpid.id AS pid,");
        hql.append("mpi.inspectionDate,");
        hql.append("mpi.modifyDatetime,");
        hql.append("mpi.partName,");
        hql.append("mpi.partNo,");
        hql.append("mpi.sn,");
        hql.append("mpi.qty,");
        hql.append("mpi.workCenterCode,");
        hql.append("mpi.createUser,");
        hql.append("mpi.modifyUser AS ncModifyUser,");
        hql.append("mpi.inspectionStatus,");
        hql.append("mpi.ncStatus,");
        hql.append("mpid.defectLocation,");
        hql.append("mpid.frontBack,");
        hql.append("mpid.defectTypeName,");
        hql.append("mm.lgort,");
        hql.append("mm.umlgo,");
        hql.append("mm.zrsum,");
        hql.append("mrt.status AS taskStatus,");
        hql.append("mrt.reworkOrder,");
        hql.append("mrt.modifyUser,");
        hql.append("mp.partTypeCode,");
        hql.append("mwc.workCenterName");
        hql.append(")");
        packHqlForQuery(hql, model);

        hql.append(" order by mpi.createDatetime asc ");
        Query query = entityManager.createQuery(hql.toString(), MesInspectionProductDataAmountReportModel.class);

        packParameterForQuery(query, model);

        query.setMaxResults(pager.getPageSize());
        if (pager.getCurrentPage() == 0) {
            pager.setCurrentPage(1);
        }
        query.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        return query.getResultList();

    }

    private void packHqlForQuery(StringBuffer hql, MesInspectionProductDataAmountReportModel model) {
        hql.append(" from MesPartInspection as mpi ");
        hql.append(" left join MesPartInspectionDetail as mpid on (mpi.id = mpid.pid and mpid.organizeCode = :organizeCode and mpid.isDeleted = :isDeleted and mpid.isValid = :isValid) ");
        hql.append(" left join MesMove as mm on (mpi.id = mm.partInspectionId and mm.organizeCode = :organizeCode and mm.isDeleted = :isDeleted and mm.isValid = :isValid) ");
        hql.append(" left join MesReworkTask as mrt on (mpi.id = mrt.partInspectionId and mrt.organizeCode = :organizeCode and mrt.isDeleted = :isDeleted and mrt.isValid = :isValid) ");
        hql.append(" left join MesPart as mp on (mpi.partNo = mp.partNo and mp.organizeCode = :organizeCode and mp.isDeleted = :isDeleted and mp.isValid = :isValid) ");
        hql.append(" left join MesWorkCenter as mwc on (mpi.workCenterCode = mwc.workCenterCode and mwc.organizeCode = :organizeCode and mwc.isDeleted = :isDeleted and mwc.isValid = :isValid) ");
        hql.append(" where mpi.organizeCode = :organizeCode ");
        hql.append(" and mpi.isValid = :isValid ");
        hql.append(" and mpi.isDeleted = :isDeleted ");
        if (!StringUtils.isEmpty(model.getInspectionDateStart())) {
            hql.append(" and mpi.inspectionDate >= :inspectionDateStart ");
        }
        if (!StringUtils.isEmpty(model.getInspectionDateEnd())) {
            hql.append(" and mpi.inspectionDate <= :inspectionDateEnd ");
        }
        if (!StringUtils.isEmpty(model.getModifyDateTimeStart())) {
            hql.append(" and mpi.modifyDatetime >= :modifyDateTimeStart ");
        }
        if (!StringUtils.isEmpty(model.getModifyDateTimeEnd())) {
            hql.append(" and mpi.modifyDatetime <= :modifyDateTimeEnd ");
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            hql.append(" and mpi.partNo like :partNo ");
        }
        if (!StringUtils.isEmpty(model.getPartName())) {
            hql.append(" and mpi.partName like :partName ");
        }
        if (!StringUtils.isEmpty(model.getProductSn())) {
            hql.append(" and mpi.sn like :productSn ");
        }
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            hql.append(" and mpi.workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(model.getCreateUser())) {
            hql.append(" and mpi.createUser like :createUser ");
        }
        if (!StringUtils.isEmpty(model.getInspectionStatus())) {
            hql.append(" and mpi.inspectionStatus = :inspectionStatus ");
        }
        if (!StringUtils.isEmpty(model.getNcStatus())) {
            hql.append(" and mpi.ncStatus = :ncStatus ");
        }
        if (!StringUtils.isEmpty(model.getDefectLocation())) {
            hql.append(" and mpid.defectLocation like :defectLocation ");
        }
        if (!StringUtils.isEmpty(model.getFrontBack())) {
            hql.append(" and mpid.frontBack = :frontBack ");
        }
        if (!StringUtils.isEmpty(model.getDefectTypeName())) {
            hql.append(" and mpid.defectTypeName like :defectTypeName ");
        }
        if (!StringUtils.isEmpty(model.getUmlgo())) {
            hql.append(" and mm.umlgo = :umlgo ");
        }
        if (!StringUtils.isEmpty(model.getTaskStatus())) {
            hql.append(" and mrt.status = :taskStatus ");
        }
        if (!StringUtils.isEmpty(model.getModifyUser())) {
            hql.append(" and mrt.modifyUser like :modifyUser ");
        }
        if (!StringUtils.isEmpty(model.getReworkOrder())) {
            hql.append(" and mrt.reworkOrder like :reworkOrder ");
        }
        if (!StringUtils.isEmpty(model.getPartTypeCode())) {
            hql.append(" and mp.partTypeCode like :partTypeCode ");
        }
    }

    private void packParameterForQuery(Query query, MesInspectionProductDataAmountReportModel model) {
        query.setParameter(MesExtConstWords.ORGANIZE_CODE, model.getOrganizeCode());
        query.setParameter(MesExtConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        if (!StringUtils.isEmpty(model.getInspectionDateStart())) {
            query.setParameter(MesExtConstWords.INSPECTION_DATE_START, model.getInspectionDateStart());
        }
        if (!StringUtils.isEmpty(model.getInspectionDateEnd())) {
            query.setParameter(MesExtConstWords.INSPECTION_DATE_END, model.getInspectionDateEnd());
        }
        if (!StringUtils.isEmpty(model.getModifyDateTimeStart())) {
            query.setParameter(MesExtConstWords.MODIFY_DATE_TIME_START, model.getModifyDateTimeStart());
        }
        if (!StringUtils.isEmpty(model.getModifyDateTimeEnd())) {
            query.setParameter(MesExtConstWords.MODIFY_DATE_TIME_END, model.getModifyDateTimeEnd());
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            query.setParameter(MesExtConstWords.PART_NO, MesExtConstWords.PERCENT + model.getPartNo() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getPartName())) {
            query.setParameter(MesExtConstWords.PART_NAME, MesExtConstWords.PERCENT + model.getPartName() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getProductSn())) {
            query.setParameter(MesExtConstWords.PRODUCT_SN, MesExtConstWords.PERCENT + model.getProductSn() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            query.setParameter(MesExtConstWords.WORK_CENTER_CODE, model.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(model.getCreateUser())) {
            query.setParameter(MesExtConstWords.CREATE_USER, MesExtConstWords.PERCENT + model.getCreateUser() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getInspectionStatus())) {
            query.setParameter(MesExtConstWords.INSPECTION_STATUS, model.getInspectionStatus());
        }
        if (!StringUtils.isEmpty(model.getNcStatus())) {
            query.setParameter(MesExtConstWords.NC_STATUS, model.getNcStatus());
        }
        if (!StringUtils.isEmpty(model.getDefectLocation())) {
            query.setParameter(MesExtConstWords.DEFECT_LOCATION, MesExtConstWords.PERCENT + model.getDefectLocation() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getFrontBack())) {
            query.setParameter(MesExtConstWords.FRONT_BACK, model.getFrontBack());
        }
        if (!StringUtils.isEmpty(model.getDefectTypeName())) {
            query.setParameter(MesExtConstWords.DEFECT_TYPE_NAME, MesExtConstWords.PERCENT + model.getDefectTypeName() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getUmlgo())) {
            query.setParameter(MesExtConstWords.UMLGO, model.getUmlgo());
        }
        if (!StringUtils.isEmpty(model.getTaskStatus())) {
            query.setParameter(MesExtConstWords.TASK_STATUS, model.getTaskStatus());
        }
        if (!StringUtils.isEmpty(model.getModifyUser())) {
            query.setParameter(MesExtConstWords.MODIFY_USER, MesExtConstWords.PERCENT + model.getModifyUser() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getReworkOrder())) {
            query.setParameter(MesExtConstWords.REWORK_ORDER, MesExtConstWords.PERCENT + model.getReworkOrder() + MesExtConstWords.PERCENT);
        }
        if (!StringUtils.isEmpty(model.getPartTypeCode())) {
            query.setParameter(MesExtConstWords.PART_TYPE_CODE, MesExtConstWords.PERCENT + model.getPartTypeCode() + MesExtConstWords.PERCENT);
        }
    }
}
