package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentSpotCheckDetailService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanMapUtilsExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheck;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentSpotCheckDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentSpotCheckRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentVariableRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesEquipmentSpotCheckDetailService extends BaseMesService<MesEquipmentSpotCheckDetail> implements IMesEquipmentSpotCheckDetailService {

    @Autowired
    private MesEquipmentSpotCheckRepository equipmentSpotCheckRepository;

    @Autowired
    private MesEquipmentVariableRepository mesEquipmentVariableRepository;

    protected void setPackQueryBean(MesEquipmentSpotCheckDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(), MesExtConstWords.PID, packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckItemCode(), "spotCheckItemCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getSpotCheckItemName(), "spotCheckItemName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getEquipCode(), MesExtConstWords.EQUIP_CODE, packBean);
        DdlPreparedPack.getNumEqualPack(bean.getDataType(), "dataType", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getSpotCheckItemType(), "spotCheckItemType", packBean);
        //43100 设备点检设置明细导入完成后，期望序列号可以正排
        packBean.setOrderByStr(" order by convert(spot_check_item_code, SIGNED) ");
    }


    @Override
    protected void onInsertBean(MesEquipmentSpotCheckDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检项代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.PID, roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckItemCode(), "spotCheckItemCode", roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检项代码【%s】已经存在，请检查数据", item.getSpotCheckItemCode())
                    .build();
        }

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.ID, packBean);
        MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckRepository.getByProperty(packBean);
        if (null == equipmentSpotCheck) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检方案ID【%s】信息不存在，请检查数据", item.getPid())
                    .build();
        }
        item.setEquipCode(equipmentSpotCheck.getEquipCode());

    }

    @Override
    protected void onUpdateBean(MesEquipmentSpotCheckDetail item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检项代码不能为空");

        DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.PID, roadPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSpotCheckItemCode(), "spotCheckItemCode", roadPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), MesExtConstWords.ID, roadPackBean);
        boolean flg = baseRDao.isExitByHql(roadPackBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检项代码【%s】已经存在，请检查数据", item.getSpotCheckItemCode())
                    .build();
        }

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.ID, packBean);
        MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckRepository.getByProperty(packBean);
        if (null == equipmentSpotCheck) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("点检方案ID【%s】信息不存在，请检查数据", item.getPid())
                    .build();
        }
        item.setEquipCode(equipmentSpotCheck.getEquipCode());

    }
    @Override
    public void validateImport(List<MesEquipmentSpotCheckDetail> beanList) {

        for (MesEquipmentSpotCheckDetail item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检项代码不能为空");

            DdlPackBean roadPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.PID, roadPackBean);
            DdlPreparedPack.getStringEqualPack(item.getSpotCheckItemCode(), "spotCheckItemCode", roadPackBean);
            boolean flg = baseRDao.isExitByHql(roadPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("点检项代码【%s】已经存在，请检查数据", item.getSpotCheckItemCode())
                        .build();
            }

            DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.ID, packBean);
            MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckRepository.getByProperty(packBean);
            if (null == equipmentSpotCheck) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("点检方案ID【%s】信息不存在，请检查数据", item.getPid())
                        .build();
            }
            item.setEquipCode(equipmentSpotCheck.getEquipCode());

        }
    }
    @Override
    public List<MesEquipmentSpotCheckDetail> validateReturnImport(List<MesEquipmentSpotCheckDetail> beanList, String equipmentCode) {
        if (CollectionUtils.isEmpty(beanList)) return beanList;
        //设备变量MAP
        Map<String, MesEquipmentVariable> variableMap = getEquipmentVariableMap(beanList, equipmentCode);
        for (MesEquipmentSpotCheckDetail item : beanList) {
            // 数据校验
            if(StringUtil.isEmpty(item.getPid())){
                ValidatorBean.checkNotNull(item.getSpotCheckCode(), "点检项目代码不能为空");
                ValidatorBean.checkNotNull(item.getSpotCheckItemCode(), "点检内容代码不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(item.getSpotCheckCode(), "spotCheckCode", seriesPackBean);
                MesEquipmentSpotCheck itemFlg = equipmentSpotCheckRepository.getByProperty(seriesPackBean);
                if (StringUtil.isEmpty(itemFlg)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目代码不存在，请检查数据", item.getSpotCheckCode())
                            .build();
                }
                item.setPid(itemFlg.getId());
                item.setEquipCode(itemFlg.getEquipCode());
            }else {
                ValidatorBean.checkNotNull(item.getId(), "点检项目id不能为空");

                DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(item.getId(), MesExtConstWords.ID, seriesPackBean);

                if (!baseRDao.isExitByHql(seriesPackBean)) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("【%s】点检项目id不存在，请检查数据", item.getId())
                            .build();
                }

                DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(item.getPid(), MesExtConstWords.ID, packBean);
                MesEquipmentSpotCheck equipmentSpotCheck = equipmentSpotCheckRepository.getByProperty(packBean);
                if (null == equipmentSpotCheck) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("点检方案ID【%s】信息不存在，请检查数据", item.getPid())
                            .build();
                }
                item.setEquipCode(equipmentSpotCheck.getEquipCode());

            }
            //实际值地址id
            if (!Objects.isNull(variableMap) && !StringUtil.isEmpty(item.getRealValueAddress()) && variableMap.containsKey(item.getRealValueAddress())) {
                item.setRealEquipVariableId(variableMap.get(item.getRealValueAddress()).getId());
                item.setRealEquipId(variableMap.get(item.getRealValueAddress()).getEquipId().longValue());
            }
            //设定值地址id
            if (!Objects.isNull(variableMap) && !StringUtil.isEmpty(item.getSetValueAddress()) && variableMap.containsKey(item.getSetValueAddress())) {
                item.setSetEquipVariableId(variableMap.get(item.getSetValueAddress()).getId());
                item.setSetEquipId(variableMap.get(item.getSetValueAddress()).getEquipId().longValue());
            }
        }
        return beanList;
    }

    private Map<String, MesEquipmentVariable> getEquipmentVariableMap(List<MesEquipmentSpotCheckDetail> beanList, String equipmentCode) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        //String organizeCode = "CK01";

        List<String> addressList = new ArrayList<>();
        //查询设备变量信息
        List<String> realAddress = beanList.stream().map(MesEquipmentSpotCheckDetail::getRealValueAddress).filter(t-> !StringUtils.isEmpty(t)).distinct().collect(Collectors.toList());
        List<String> setAddress = beanList.stream().map(MesEquipmentSpotCheckDetail::getSetValueAddress).filter(t-> !StringUtils.isEmpty(t)).distinct().collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(realAddress)) addressList.addAll(realAddress);
        if (!CollectionUtils.isEmpty(setAddress)) addressList.addAll(setAddress);
        if (CollectionUtils.isEmpty(addressList)) return null;


        Map<String, MesEquipmentVariable> beanMap = new HashMap();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(addressList, "tagName", packBean);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", packBean);
        List<MesEquipmentVariable> baseDataList = mesEquipmentVariableRepository.findByHqlWhere(packBean);

        if (!CollectionUtils.isEmpty(baseDataList)) {
            beanMap =  baseDataList.stream().collect(Collectors.toMap(mesEquipmentVariable -> mesEquipmentVariable.getTagName(), o-> o));
        } else {
            MesException.throwMesBusiException(addressList + "信息不存在");
        }
        return beanMap;
    }
}
