package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.board;

import cn.estsh.i3plus.ext.mes.api.busi.board.IMesYfBoardService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.board.IMesYfBoardDao;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesGepicsInspectionInfoModel;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesWorkCenterBoardConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.model.board.MesWorkCenterBoardResultModel;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.edi.wh.order.MesCimGmGepics;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.repository.wh.MesCimGmGepicsRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import com.xxl.job.core.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 看板-业务方法实现
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/19 11:08
 * @Modify:
 **/
@Service
public class IMesYfBoardServiceImpl implements IMesYfBoardService {

    @Autowired
    private IMesYfBoardDao mesYfBoardDao;

    @Autowired
    private MesDowntimeRecordRepository mesDowntimeRecordRepository;

    @Autowired
    private MesWorkOrderRepository mesWorkOrderRepository;

    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;

    @Autowired
    private MesEquipmentSpotCheckRepository mesEquipmentSpotCheckRepository;

    @Autowired
    private MesSpotCheckOrderRepository mesSpotCheckOrderRepository;

    @Autowired
    private MesCimGmGepicsRepository mesCimGmGepicsRepository;

    @Autowired
    private MesDowntimeRecordRepository mesDowntimeRecordRDao;


    private static final String START_TIME = "START_TIME";

    private static final String END_TIME = "END_TIME";

    private static final Integer NUM = 12;

    /**
     * 产线看板
     */
    @Override
    public List<MesWorkCenterBoardResultModel> queryWorkCenterStatus(MesWorkCenterBoardConditionModel conditionModel) {
        List<MesWorkCenterBoardResultModel> resultModelList = new ArrayList<>();
        List<String> workCenterCodeList = conditionModel.getWorkCenterCodeList();
        //获取查询时间区间
        Map<String, Map<String, String>> timeZoneMap = getTimeZoneMap(workCenterCodeList,
                conditionModel.getTimeRangeIdentification(), conditionModel.getOrganizeCode());
        //查询当前时间区间内中的选中产线最新的的开停机记录
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
        for (String workCenterCode : timeZoneMap.keySet()) {
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            DdlPreparedPack.timeBuilder(timeZoneMap.get(workCenterCode).get(START_TIME),
                    timeZoneMap.get(workCenterCode).get(END_TIME), "createDatetime", ddlPackBean, true);
            DdlPreparedPack.getOrderByPack(
                    new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, ddlPackBean);
            int queryCount = mesDowntimeRecordRepository.findByHqlWhereCount(ddlPackBean);
            //封装页面展示数据
            MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
            resultModel.setWorkCenterCode(workCenterCode);
            if (queryCount == 0) {
                resultModel.setWorkCenterCodeStatus(MesExtEnumUtil.BOARD_WORK_CENTER_STATUS.NORMAL.getValue());
            }else {
                resultModel.setWorkCenterCodeStatus(MesExtEnumUtil.BOARD_WORK_CENTER_STATUS.ABNORMAL.getValue());
            }
            resultModelList.add(resultModel);
        }
        return resultModelList;
    }

    @Override
    public MesWorkCenterBoardResultModel queryWorkCenterProductionCompletionRate(MesWorkCenterBoardConditionModel conditionModel) {
        Double workCenterCompletedQuantity = 0D;
        Double workCenterPlannedQuantity = 0D;
        //获取查询时间区间
        Map<String, Map<String, String>> timeZoneMap = getTimeZoneMap(conditionModel.getWorkCenterCodeList(),
                conditionModel.getTimeRangeIdentification(), conditionModel.getOrganizeCode());
        //查询工单
        List<MesWorkOrder> mesWorkOrderList = queryMesWorkOrder(timeZoneMap, conditionModel);
        //封装汇总数据
        MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
        for (MesWorkOrder mesWorkOrder : mesWorkOrderList) {
            workCenterCompletedQuantity = workCenterCompletedQuantity + mesWorkOrder.getCompleteQty();
            workCenterPlannedQuantity = workCenterPlannedQuantity + mesWorkOrder.getQty();
        }
        resultModel.setWorkCenterCompletedQuantity(workCenterCompletedQuantity);
        resultModel.setWorkCenterPlannedQuantity(workCenterPlannedQuantity);
        double workCenterCompletionRate;
        if (workCenterCompletedQuantity == 0 || workCenterPlannedQuantity == 0) {
            workCenterCompletionRate = 0;
        }else {
            workCenterCompletionRate = workCenterCompletedQuantity * 100 / workCenterPlannedQuantity;
        }
        BigDecimal bd = new BigDecimal(workCenterCompletionRate);
        bd = bd.setScale(2, RoundingMode.HALF_UP);
        resultModel.setWorkCenterCompletionRate(bd);
        return resultModel;
    }

    @Override
    public List<MesWorkCenterBoardResultModel> queryWorkCenterWorkOrderCompletionStatus(MesWorkCenterBoardConditionModel conditionModel) {
        List<MesWorkCenterBoardResultModel> resultModelList = new ArrayList<>();
        //获取查询时间区间
        Map<String, Map<String, String>> timeZoneMap = getTimeZoneMap(conditionModel.getWorkCenterCodeList(),
                conditionModel.getTimeRangeIdentification(), conditionModel.getOrganizeCode());
        //查询工单
        List<MesWorkOrder> mesWorkOrderList = queryMesWorkOrder(timeZoneMap, conditionModel);
        if (!CollectionUtils.isEmpty(mesWorkOrderList)) {
            Set<String> workOrderNoSet = mesWorkOrderList.stream().map(MesWorkOrder::getWorkOrderNo).collect(Collectors.toSet());
            //查询当前产线在指定时间段内加工完成的工单条码数据
            List<MesWorkCenterBoardResultModel> mesProductOffLineQueryResultList = mesYfBoardDao.queryCompletedSnInfo(
                    conditionModel.getWorkCenterCodeList().get(0),
                    conditionModel.getOrganizeCode(),
                    timeZoneMap.get(conditionModel.getWorkCenterCodeList().get(0)).get(START_TIME),
                    timeZoneMap.get(conditionModel.getWorkCenterCodeList().get(0)).get(END_TIME),
                    workOrderNoSet);
            //查询结果按照工单号分组
            Map<String, List<MesWorkCenterBoardResultModel>> workOrderAndSnDataByOrderNoMap =
                    mesProductOffLineQueryResultList.stream().collect(Collectors.groupingBy(MesWorkCenterBoardResultModel::getWorkOrderNo));
            //封装汇总数据
            for (MesWorkOrder mesWorkOrder : mesWorkOrderList) {
                MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
                resultModel.setWorkOrderNo(mesWorkOrder.getWorkOrderNo());
                resultModel.setPartNo(mesWorkOrder.getPartNo());
                resultModel.setPartDescription(mesWorkOrder.getPartName());
                resultModel.setWorkOrderCompletedQuantity(mesWorkOrder.getCompleteQty());
                resultModel.setWorkOrderPlannedQuantity(mesWorkOrder.getQty());
                //计算当前工单的合格率
                double workOrderFirstPassRate;
                List<MesWorkCenterBoardResultModel> resultModels = workOrderAndSnDataByOrderNoMap.get(mesWorkOrder.getWorkOrderNo());
                if (CollectionUtils.isEmpty(resultModels)) {
                    workOrderFirstPassRate = 100;
                }else {
                    //合格数
                    long count = resultModels.stream().filter(model -> model.getQcStatus() == MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()).count();
                    workOrderFirstPassRate = count * 100 / resultModels.size();
                    if (workOrderFirstPassRate > 100) {
                        workOrderFirstPassRate = 100;
                    }
                }
                BigDecimal bd = new BigDecimal(workOrderFirstPassRate);
                bd = bd.setScale(2, RoundingMode.HALF_UP);
                resultModel.setWorkOrderFirstPassRate(bd);
                resultModelList.add(resultModel);
            }
        }
        return resultModelList;
    }

    @Override
    public List<MesWorkCenterBoardResultModel> queryUnitHourCompletionRate(MesWorkCenterBoardConditionModel conditionModel) {
        List<MesWorkCenterBoardResultModel> resultModelList = new ArrayList<>();
        //查询当前产线设置的目标值
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(conditionModel.getWorkCenterCodeList().get(0), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRepository.getByProperty(ddlPackBean);
        String jph;
        if (Objects.isNull(mesWorkCenter) || StringUtils.isEmpty(mesWorkCenter.getJph())) {
            jph = "0";
        }else {
            jph = mesWorkCenter.getJph();
        }
        //获取并汇总当前产线在当前时间前12个小时的每个小时时间段内工单的完成数
        Map<String, Map<String, String>> timeZoneMap = getLastHoursTimeZoneMap(NUM);
        //查询当前产线12小时内的生产汇报完成的数据
        List<MesWorkCenterBoardResultModel> mesProductOffLineQueryResultList = mesYfBoardDao.queryMesProductOffLineModel(
                        conditionModel.getWorkCenterCodeList().get(0),
                        conditionModel.getOrganizeCode(),
                        timeZoneMap.get(START_TIME).get(START_TIME),
                        timeZoneMap.get(END_TIME).get(END_TIME));
        for (String timeZone : timeZoneMap.keySet()) {
            MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
            if (START_TIME.equals(timeZone) || END_TIME.equals(timeZone)) {
                continue;
            }
            //横坐标赋值
            resultModel.setUnitHourCompletionRateTimePeriod(timeZone);
            //目标值
            resultModel.setUnitHourCompletionRateTargetValue(Double.valueOf(jph));
            if (CollectionUtils.isEmpty(mesProductOffLineQueryResultList)) {
                resultModel.setUnitHourCompletionRate(0);
            }else {
                //筛选生产汇报数据中在当前时间区段中的产品条码数
                List<MesWorkCenterBoardResultModel> nowResultList = mesProductOffLineQueryResultList.stream().filter(model -> (
                        StringUtils.compare(model.getCreateDateTime(), timeZoneMap.get(timeZone).get(START_TIME)) > 0
                                && StringUtils.compare(timeZoneMap.get(timeZone).get(END_TIME), model.getCreateDateTime()) > 0
                )).collect(Collectors.toList());
                resultModel.setUnitHourCompletionRate(nowResultList.size());
            }
            resultModelList.add(resultModel);
        }
        return resultModelList;
    }

    @Override
    public List<MesWorkCenterBoardResultModel> queryWorkCenterFirstTimeOfflineQualificationRate(MesWorkCenterBoardConditionModel conditionModel) {
        //获取并汇总当前时间前12个小时的每个小时时间段内数据（最高100）
        //合格数 / 加工完成（包含已汇报，即工单的完成数；以及待汇报的）
        List<MesWorkCenterBoardResultModel> resultModelList = new ArrayList<>();
        Map<String, Map<String, String>> timeZoneMap = getLastHoursTimeZoneMap(NUM);
        //查询当前产线在指定时间段内加工完成的条码数据
        List<MesWorkCenterBoardResultModel> mesProductOffLineQueryResultList = mesYfBoardDao.queryCompletedSnInfo(
                conditionModel.getWorkCenterCodeList().get(0),
                conditionModel.getOrganizeCode(),
                timeZoneMap.get(START_TIME).get(START_TIME),
                timeZoneMap.get(END_TIME).get(END_TIME),
                null);
        //依次汇总每个时间段的数据
        for (String timeZone : timeZoneMap.keySet()) {
            MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
            if (START_TIME.equals(timeZone) || END_TIME.equals(timeZone)) {
                continue;
            }
            //横坐标赋值
            resultModel.setWorkCenterFirstTimeOfflineQualificationRateTimePeriod(timeZone);
            //筛选当前时间段的生产加工记录
            List<MesWorkCenterBoardResultModel> nowResultList = mesProductOffLineQueryResultList.stream().filter(model -> (
                    StringUtils.compare(model.getCreateDateTime(), timeZoneMap.get(timeZone).get(START_TIME)) > 0
                            && StringUtils.compare(timeZoneMap.get(timeZone).get(END_TIME), model.getCreateDateTime()) > 0
            )).collect(Collectors.toList());
            Map<Integer, List<MesWorkCenterBoardResultModel>> resultModelByQcStatusMap = nowResultList.stream().
                    collect(Collectors.groupingBy(MesWorkCenterBoardResultModel::getQcStatus));
            if (CollectionUtils.isEmpty(nowResultList)
                    || !resultModelByQcStatusMap.containsKey(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue())) {
                //当前时间段没有生产加工记录或者没有合格的生产加工条码时返回值置null（保证数据为0时瀑布图连线的垂直）
                resultModel.setWorkCenterFirstTimeOfflineQualificationRate(null);
            }else {
                double workCenterFirstTimeOfflineQualificationRate = resultModelByQcStatusMap.get(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()).size() * 100 / nowResultList.size();
                //最大值只允许100
                if (workCenterFirstTimeOfflineQualificationRate > 100) {
                    workCenterFirstTimeOfflineQualificationRate = 100;
                }
                BigDecimal bd = new BigDecimal(workCenterFirstTimeOfflineQualificationRate);
                bd = bd.setScale(2, RoundingMode.HALF_UP);
                resultModel.setWorkCenterFirstTimeOfflineQualificationRate(bd);
            }
            resultModelList.add(resultModel);
        }
        return resultModelList;
    }

    @Override
    public List<MesWorkCenterBoardResultModel> queryInspectionResults(MesWorkCenterBoardConditionModel conditionModel) {
        String workCenterCode = conditionModel.getWorkCenterCodeList().get(0);
        List<MesWorkCenterBoardResultModel> resultModelList = new ArrayList<>();
        //查询当前产线配置的点检类型
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
        List<MesEquipmentSpotCheck> mesEquipmentSpotCheckList = mesEquipmentSpotCheckRepository.findByHqlWhere(ddlPackBean);
        Set<Integer> spotCheckOrderTypeSet = mesEquipmentSpotCheckList.
                stream().map(MesEquipmentSpotCheck::getSpotCheckOrderType).collect(Collectors.toSet());
        //获取查询时间区间
        Map<String, Map<String, String>> timeZoneMap = getTimeZoneMap(conditionModel.getWorkCenterCodeList(),
                conditionModel.getTimeRangeIdentification(), conditionModel.getOrganizeCode());
        //查询当前产线在指定时间区间内的点检单记录并按照点检类型进行分组
        ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
        DdlPreparedPack.timeBuilder(timeZoneMap.get(workCenterCode).get(START_TIME),
                timeZoneMap.get(workCenterCode).get(END_TIME), "createDatetime", ddlPackBean, true);
        List<MesSpotCheckOrder> mesSpotCheckOrderList = mesSpotCheckOrderRepository.findByHqlWhere(ddlPackBean);
        Map<Integer, List<MesSpotCheckOrder>> mesSpotCheckOrderByTypeMap = mesSpotCheckOrderList.
                stream().collect(Collectors.groupingBy(MesSpotCheckOrder::getSpotCheckOrderType));
        //遍历“点检类型”枚举
        for (MesExtEnumUtil.EQUIPMENT_CHECK_ORDER_TYPE typeEnum : MesExtEnumUtil.EQUIPMENT_CHECK_ORDER_TYPE.values()) {
            if (typeEnum.getValue() == 6) {
                    //不添加工装点检
                    continue;
            }
            MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
            //截取前两位作为标题
            resultModel.setInspectionItems(typeEnum.getDescription().substring(0, 2));
            //判断点检结果
            if (spotCheckOrderTypeSet.contains(typeEnum.getValue())) {
                //配置该种点检类型
                if (mesSpotCheckOrderByTypeMap.containsKey(typeEnum.getValue())) {
                    //存在该种点检类型的点检单数据
                    List<MesSpotCheckOrder> mesSpotCheckOrders = mesSpotCheckOrderByTypeMap.get(typeEnum.getValue());
                    //按修改时间降序
                    mesSpotCheckOrders.sort(Comparator.comparing(MesSpotCheckOrder::getModifyDatetime).reversed());
                    if (MesExtEnumUtil.SPOT_CHECK_ORDER_RESULT_TYPE.OK.getValue() == mesSpotCheckOrders.get(0).getSpotCheckOrderResult()) {
                        //最新的点检单结果为 OK
                        //结果设置为“正常”
                        resultModel.setInspectionResults(MesExtEnumUtil.INSPECTION_RESULTS.NORMAL.getValue());
                    }else if (MesExtEnumUtil.SPOT_CHECK_ORDER_RESULT_TYPE.NOK.getValue() == mesSpotCheckOrders.get(0).getSpotCheckOrderResult()) {
                        //最新的点检单结果为 NOK
                        //结果设置为“异常”
                        resultModel.setInspectionResults(MesExtEnumUtil.INSPECTION_RESULTS.ABNORMAL.getValue());
                    }else {
                        //其它结果设置为“异常”
                        resultModel.setInspectionResults(MesExtEnumUtil.INSPECTION_RESULTS.ABNORMAL.getValue());
                    }
                }else {
                    //不存在该种点检类型的点检单数据
                    //结果设置为“未做”
                    resultModel.setInspectionResults(MesExtEnumUtil.INSPECTION_RESULTS.NOT_DONE.getValue());
                }
            }else {
                //未配置该种点检类型
                //结果设置为“不涉及”
                resultModel.setInspectionResults(MesExtEnumUtil.INSPECTION_RESULTS.NOT_INVOLVED.getValue());
            }
            resultModelList.add(resultModel);
        }
        return resultModelList;
    }

    @Override
    public MesWorkCenterBoardResultModel queryAbnormalDowntime(MesWorkCenterBoardConditionModel conditionModel) {
        MesWorkCenterBoardResultModel resultModel = new MesWorkCenterBoardResultModel();
        //获取查询时间区间（需求未明确，暂时注释，返回固定值）
        Map<String, Map<String, String>> timeZoneMap = getTimeZoneMap(conditionModel.getWorkCenterCodeList(),
                conditionModel.getTimeRangeIdentification(), conditionModel.getOrganizeCode());

        if(!timeZoneMap.isEmpty()){
            Set<String> keySet = timeZoneMap.keySet();
            for (String key : keySet) {
                Map<String, String> map = timeZoneMap.get(key);
                String startTime = map.get("START_TIME");
                String endTime = map.get("END_TIME");
                //根据产线查询停机记录表汇总
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
                DdlPreparedPack.getStringBiggerPack(startTime,"createDatetime",ddlPackBean);
                DdlPreparedPack.getStringSmallerPack(endTime,"createDatetime",ddlPackBean);
                DdlPreparedPack.getInPackList(conditionModel.getWorkCenterCodeList(), "workCenterCode", ddlPackBean);
                List<MesDowntimeRecord> recordList = mesDowntimeRecordRDao.findByHqlWhere(ddlPackBean);
                //遍历recordList，汇总停机时间
                long total = 0L;
                for (MesDowntimeRecord record : recordList) {
                    // 如果停机结束时间还在，则需要补充动态停机时间，否则需要有停机时间
                    if (StringUtils.isEmpty(record.getModifyDatetime())) {
                        Date startTDateTime = DateUtil.parseDateTime(record.getCreateDatetime());
                        Date currentTime = new Date();
                        total += (currentTime.getTime() - startTDateTime.getTime())/(1000 * 60);
                    } else {
                        Date startTDateTime = DateUtil.parseDateTime(record.getCreateDatetime());
                        Date endDateTime = DateUtil.parseDateTime(record.getModifyDatetime());
                        total +=  (endDateTime.getTime() - startTDateTime.getTime())/(1000 * 60);
                    }
                }
                if (total > 0){
                    //转换成小时分钟
                    long minutes = total % 60;
                    long hours = total / 60;
                    resultModel.setAbnormalDowntime(String.format("%02d",hours)+":"+String.format("%02d",minutes));
                }
            }
        }else {
            resultModel.setAbnormalDowntime("00:00");
        }


        return resultModel;
    }

    @Override
    public ListPager<MesGepicsInspectionInfoModel> queryGepicsInspectionInfo(MesGepicsInspectionInfoModel infoModel, Pager pager) {
        //查询数据
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(infoModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(infoModel.getGaRule(), "gaRule", packBean);
        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.VALID, "status", packBean);
        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.VALID, "type", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        pager = PagerHelper.getPager(pager, mesCimGmGepicsRepository.findByHqlWhereCount(packBean));
        List<MesCimGmGepics> mesCimGmGepicsList = mesCimGmGepicsRepository.findByHqlWherePage(packBean, pager);
        if(CollectionUtils.isEmpty(mesCimGmGepicsList)){
            return new ListPager<>(new ArrayList<>(),pager);
        }
        //数据组装
        List<MesGepicsInspectionInfoModel> modelList = new ArrayList<>(mesCimGmGepicsList.size());
        mesCimGmGepicsList.forEach(item -> modelList.add(new MesGepicsInspectionInfoModel(item)));
        return new ListPager<>(modelList,pager);
    }

    /**
     * 获取看板查询时间区段
     * @param timeRangeIdentification 显示维度
     * @return 当前显示维护对应的时间区段
     */
    private Map<String, Map<String, String>> getTimeZoneMap(List<String> workCenterCodeList, Integer timeRangeIdentification, String organizeCode) {
        Map<String, Map<String, String>> timeZoneMap = new HashMap<>();
        String startTime;
        String endTime;
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd ");
        DateTimeFormatter dayTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        //当前时间
        LocalDateTime nowTime = LocalDateTime.now();
        //当前年月日
        String nowDay = dayFormatter.format(nowTime);
        //当前年月日 时分秒
        String nowDayTime = dayTimeFormatter.format(nowTime);
        //判断当前查询的时间段（当班或者当日）
        if (MesExtEnumUtil.TIME_RANGE_IDENTIFICATION.ON_DUTY.getValue() == timeRangeIdentification) {
            //当班
            //先查询用户指定的产线在当前时间节点有效的班次信息
            List<MesShift> mesShiftList = mesYfBoardDao.queryMesShift(workCenterCodeList, organizeCode, nowDayTime);
            if (CollectionUtils.isEmpty(mesShiftList)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail(workCenterCodeList.toString() + " 产线未维护有效的班次信息！")
                        .build();
            }
            Map<String, List<MesShift>> shiftByWorkCenterCodeMap = mesShiftList.stream().collect(Collectors.groupingBy(MesShift::getWorkCenterCode));
            //依次处理每个产线对应的班次数据，获取与当前时间相符的班次
            Map<String, MesShift> passShiftByWorkCenterCodeMap = new HashMap<>();
            for (String workCenterCode : workCenterCodeList) {
                if (shiftByWorkCenterCodeMap.containsKey(workCenterCode)) {
                    List<MesShift> mesShifts = shiftByWorkCenterCodeMap.get(workCenterCode);
                    for (MesShift mesShift : mesShifts) {
                        //上班时间
                        String workTime = nowDay + mesShift.getStartTime();
                        //下班时间
                        String closingTime;
                        if (StringUtils.compare(mesShift.getStartTime(), mesShift.getEndTime()) > 0) {
                            //跨天班次，下班时间的日期需要加一
                            closingTime = dayFormatter.format(nowTime.plusDays(1)) + mesShift.getEndTime();
                        }else {
                            //当天班次
                            closingTime = nowDay + mesShift.getEndTime();
                        }
                        if (StringUtils.compare(nowDayTime, workTime) > 0 && StringUtils.compare(closingTime, nowDayTime) > 0) {
                            //当前时间点在当前班次的上下班区间内
                            mesShift.setStartTime(workTime);
                            mesShift.setEndTime(closingTime);
                            passShiftByWorkCenterCodeMap.put(workCenterCode, mesShift);
                        }
                    }
                }else {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                            .setErrorDetail(workCenterCode + " 产线未维护有效的班次信息！")
                            .build();
                }
            }
            //获取最终的班次时间区间
            for (String workCenterCode : passShiftByWorkCenterCodeMap.keySet()) {
                MesShift mesShift = passShiftByWorkCenterCodeMap.get(workCenterCode);
                Map<String, String> map = new HashMap<>();
                map.put(START_TIME, mesShift.getStartTime());
                map.put(END_TIME, mesShift.getEndTime());
                timeZoneMap.put(workCenterCode, map);
            }
        }else if (MesExtEnumUtil.TIME_RANGE_IDENTIFICATION.ON_THE_SAME_DAY.getValue() == timeRangeIdentification) {
            Map<String, String> timeMap = new HashMap<>();
            //当日
            startTime = nowDay + "00:00:00";
            endTime = nowDay + "23:59:59";
            timeMap.put(START_TIME, startTime);
            timeMap.put(END_TIME, endTime);
            for (String workCenterCode : workCenterCodeList) {
                timeZoneMap.put(workCenterCode, timeMap);
            }
        }else {
            //显示维度格式有误，直接报错
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.PARAMETER_EXCEPTION.getCode())
                    .setErrorDetail("显示维护传参有误！")
                    .build();
        }
        return timeZoneMap;
    }

    /**
     * 查询工单
     * @param timeZoneMap 时间区间
     * @param conditionModel 查询条件
     * @return 工单数据
     */
    private List<MesWorkOrder> queryMesWorkOrder(Map<String, Map<String, String>> timeZoneMap, MesWorkCenterBoardConditionModel conditionModel) {
        String workCenterCode = conditionModel.getWorkCenterCodeList().get(0);
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(conditionModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
        //计划开始时间
        DdlPreparedPack.timeBuilder(timeZoneMap.get(workCenterCode).get(START_TIME),
                timeZoneMap.get(workCenterCode).get(END_TIME), "planStartTime", ddlPackBean, true);
        List<MesWorkOrder> workOrderList = mesWorkOrderRepository.findByHqlWhere(ddlPackBean);
        //剔除P单数据
        workOrderList = CollectionUtils.isEmpty(workOrderList) ? null :
                workOrderList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getOrderFlag()) &&
                        !o.getOrderFlag().equals(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.P.getValue()))).collect(Collectors.toList());
        return workOrderList;
    }

    /**
     * 获取当前时间前指定小时数内的每个小时区间的开始与结束时间
     * @param designatedQuantity 指定小时之前
     * @return 当前时间前指定小时数内的每个小时区间的开始与结束时间
     */
    private Map<String, Map<String, String>> getLastHoursTimeZoneMap(Integer designatedQuantity) {
        Map<String, Map<String, String>> timeZoneMap = new LinkedHashMap<>();
        DateTimeFormatter hourFormatter = DateTimeFormatter.ofPattern("HH");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd ");
        LocalDateTime nowTime = LocalDateTime.now();
        String nowDay = dayFormatter.format(nowTime);
        String startTime;
        String endTime;
        String key;
        for (int i = 0; i < designatedQuantity; i++) {
            Map<String, String> zoneMap = new HashMap<>();
            if (i == 0) {
                //第一个区段获取当前时间所在的小时区间
                startTime = hourFormatter.format(nowTime.minusHours(i)) + ":00";
                endTime = hourFormatter.format(nowTime.plusHours(1)) + ":00";
            }else {
                startTime = hourFormatter.format(nowTime.minusHours(i)) + ":00";
                endTime = hourFormatter.format(nowTime.minusHours(i - 1)) + ":00";
            }
            key = startTime + "-" + endTime;
            zoneMap.put(START_TIME, nowDay + startTime + ":00");
            zoneMap.put(END_TIME, nowDay + endTime + ":00");
            timeZoneMap.put(key, zoneMap);
            Map<String, String> map = new HashMap<>();
            if (i == 0) {
                //将第一个时间段作为本段查询时间区间的结束（因为是递减）
                map.put(END_TIME, nowDay + startTime + ":00");
                timeZoneMap.put(END_TIME, map);
            }
            if (i == designatedQuantity - 1) {
                //将最后一个时间段作为本段查询时间区间的开始（因为是递减）
                map.put(START_TIME, nowDay + endTime + ":00");
                timeZoneMap.put(START_TIME, map);
            }
        }
        //倒序处理
        Map<String, Map<String, String>> collectResult = new LinkedHashMap<>();
        ListIterator<Map.Entry> i = new ArrayList<Map.Entry>(timeZoneMap.entrySet()).listIterator(timeZoneMap.size());
        while (i.hasPrevious()) {
            Map.Entry entry = i.previous();
            collectResult.put(String.valueOf(entry.getKey()), (HashMap<String, String>) entry.getValue());
        }
        return collectResult;
    }

}
