package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.busi.IMesSortRuleCfgOfflineService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesBomService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.offline.MesProdRuleSortCfgOffline;
import cn.estsh.i3plus.pojo.mes.bean.offline.MesPullingOrderInfoOffline;
import cn.estsh.i3plus.pojo.mes.bean.offline.MesPullingOrderPartInfoOffline;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.repository.offline.MesProdRuleSortCfgOfflineRepository;
import cn.estsh.i3plus.pojo.mes.repository.offline.MesPullingOrderInfoOfflineRepository;
import cn.estsh.i3plus.pojo.mes.repository.offline.MesPullingOrderPartInfoOfflineRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class MesSortRuleCfgOfflineServiceImpl implements IMesSortRuleCfgOfflineService {

    @Autowired
    private MesProdRuleSortCfgOfflineRepository cfgOfflineRao;

    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;

    @Autowired
    private MesBomService mesBomService;

    @Autowired
    private MesBomRepository bomRao;

    @Autowired
    private MesPartSapRepository mesPartSapRao;

    @Autowired
    private MesProdRuleSortCfgRepository mesProdRuleSortCfgRao;

    @Autowired
    private MesCustomerPartRepository customerPartRao;

    @Autowired
    private MesCustomerCarModelRepository carModelRao;

    @Autowired
    private MesPullingOrderInfoOfflineRepository partPullOrderOfflineRao;

    @Autowired
    private MesPullingOrderPartInfoOfflineRepository partPullOrderDetailOfflineRao;

    @Autowired
    private MesPartSapRepository partSapRao;

    @Autowired
    private IMesConfigService mesConfigService;

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    public void deleteSortRuleOffline(String organizeCode) {
        cfgOfflineRao.deleteByProperty("organizeCode", organizeCode);
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    public void doCreateSortRuleOffline(String custPartNo, String erpPartNo, String productPartNo, String organizeCode, MesPartProdGroup mesPartProdGroup, MesPartProdGroupDetail detail) {
        /**
         * 校验 cfgOfflineRao表中是否已经存在了客户零件号，如果存在则跳过
         * 防止多个零件生产组生成相同的数据
         */
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesPartProdGroup.getWorkCenterCode(),"workCenterCode",ddlPackBean);
        int count = cfgOfflineRao.findByHqlWhereCount(ddlPackBean);
        if (count > 0){
            return;
        }
        MesProductVersion mesProductVersion= null;
        if (mesPartProdGroup.getEnableBomVersion() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()){
            mesProductVersion = getMesProductVersion(productPartNo, organizeCode);
        }
        //4.总成零件号打散bom
        String bomVersion = null;
        if (mesProductVersion != null){
            bomVersion = mesProductVersion.getAlternativePartList();
        }
        MesBom bom = mesBomService.findBom(organizeCode, bomVersion, productPartNo);
        if (bom == null) {
            return;
        }
        //5.5 获取客户零件号对应的车型配置
        /**
         * 客户零件号 对应的 erp零件号 不带z
         */
        MesCustomerPart mesCustomerPart = getMesCustomerPart(custPartNo, erpPartNo, organizeCode);
        if (mesCustomerPart == null) {
            return;
        }
        //5.打散的bom 获取排序加工规则
        List<MesWorkOrderPart> workOrderPartList = dismantleBom(productPartNo, 1, TimeTool.getNowTime(true), bom.getBomCode(), organizeCode, detail.getWorkCellCode());
       if (workOrderPartList.isEmpty()){
           return;
       }
        //6.装配件获取的三种条件
        String custCode = mesCustomerPart.getCustCode();
        MesCustomerCarModel mesCustomerCarModel = getMesCustomerCarModel(custPartNo, organizeCode, custCode);
        if (mesCustomerCarModel == null) {
            return;
        }
        //5.6 获取零件信息
        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productPartNo,"partNo", partPackBean);
        List<MesPartSap> partSapList = partSapRao.findByHqlWhere(partPackBean);

        //7.获取车型
        // 客户车型配置
//        //4.查询车型配置中的车型编号就是客户零件号 custPartNo,custPlantCode,CustCode,Plant
//        DdlPackBean carModelPackBean = DdlPackBean.getDdlPackBean(organizeCode);
//        DdlPreparedPack.getStringEqualPack(custPartNo,"custPartNo",carModelPackBean);
//        DdlPreparedPack.getStringEqualPack(organizeCode,"organizeCode",carModelPackBean);
//        List<MesCustomerCarModel> carModels = carModelRao.findByHqlWhere(carModelPackBean);


        String carModelCode = mesCustomerCarModel.getCarModelCode();
        List<MesWorkOrderAssembly> assemblyList = getAssemblyList(detail.getWorkCenterCode(), carModelCode, workOrderPartList, mesPartProdGroup);
        //7.保存到离线表中
        for (MesWorkOrderAssembly orderAssembly : assemblyList) {
            MesProdRuleSortCfgOffline cfgOffline = new MesProdRuleSortCfgOffline();
            BeanUtils.copyProperties(orderAssembly, cfgOffline);
            if (partSapList != null) {
                cfgOffline.setPartName(partSapList.get(0).getPartName());
            }
            cfgOffline.setCustPartNo(custPartNo);
            cfgOffline.setCustPartName(mesCustomerPart.getCustPartName());
            cfgOffline.setPartNo(productPartNo);
            cfgOffline.setRemark(mesCustomerCarModel.getCarModelName());
            cfgOffline.setOrganizeCode(organizeCode);
            ConvertBean.saveOrUpdate(cfgOffline, "OFFLINE-JOB");
            cfgOfflineRao.insert(cfgOffline);
        }
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    public void deletePullOffline(String organizeCode) {
        partPullOrderOfflineRao.deleteByProperty("organizeCode", organizeCode);
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    public void deletePullDetailOffline(String organizeCode) {
        partPullOrderDetailOfflineRao.deleteByProperty("organizeCode", organizeCode);
    }

    @Override
    public void doCreatePullOffline(String custPartNo, String erpPartNo,String productPartNo, String organizeCode, MesPartProdGroup mesPartProdGroup, MesPartProdGroupDetail detail, MesPartPull partPull, List<MesPartPullDetail> pullDetailList) {
        MesProductVersion mesProductVersion = getMesProductVersion(productPartNo, organizeCode);
        //4.总成零件号打散bom
        String bomVersion = mesProductVersion.getAlternativePartList();
        MesBom bom = mesBomService.findBom(organizeCode, bomVersion, productPartNo);
        if (bom == null) {
            return;
        }
        //5.5 获取客户零件号对应的车型配置
        MesCustomerPart mesCustomerPart = getMesCustomerPart(custPartNo, erpPartNo, organizeCode);
        if (mesCustomerPart == null) {
            return;
        }
        //5.6 获取零件信息
        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productPartNo,"partNo", partPackBean);
        List<MesPartSap> partSapList = partSapRao.findByHqlWhere(partPackBean);

        //5.打散的bom 获取排序加工规则
        List<MesWorkOrderPart> workOrderPartList = dismantleBom(productPartNo, 1, TimeTool.getNowTime(true), bom.getBomCode(), organizeCode, detail.getWorkCellCode());

        //6.过滤出需要拉动的零件号
        List<String> partPullDetailPartNoList = workOrderPartList.stream().map(MesWorkOrderPart::getPartNo).collect(Collectors.toList());
        Map<String, MesWorkOrderPart> orderPartMap = workOrderPartList.stream().collect(Collectors.toMap(MesWorkOrderPart::getPartNo, Function.identity()));
        List<MesPartPullDetail> pullDetails = pullDetailList.stream().filter(item -> partPullDetailPartNoList.contains(item.getPartNo())).collect(Collectors.toList());
        //7.生成拉动主单模板 MesPullingOrderPartInfoOfflineRepository
        MesPullingOrderInfoOffline mesPartPullOffline = new MesPullingOrderInfoOffline();
        if (partSapList != null) {
            mesPartPullOffline.setPartName(partSapList.get(0).getPartName());
        }
        mesPartPullOffline.setPullCode(partPull.getPullCode());
        mesPartPullOffline.setCustPartNo(custPartNo);
        mesPartPullOffline.setPartNo(productPartNo);
        mesPartPullOffline.setWorkCenterCode(partPull.getWorkCenterCode());
        mesPartPullOffline.setOrganizeCode(organizeCode);
        ConvertBean.saveOrUpdate(mesPartPullOffline,"SPS-OFFLINE");
        partPullOrderOfflineRao.insert(mesPartPullOffline);
        //8.生成拉动单详情
        for (MesPartPullDetail pullDetail : pullDetails) {
            MesPullingOrderPartInfoOffline detailOffline = new MesPullingOrderPartInfoOffline();
            detailOffline.setPid(mesPartPullOffline.getId());
            detailOffline.setPullCode(partPull.getPullCode());
            detailOffline.setCustPartNo(custPartNo);
            detailOffline.setAssPartNo(productPartNo);
            detailOffline.setOrganizeCode(organizeCode);
            detailOffline.setPartNo(pullDetail.getPartNo());
            detailOffline.setPartName(pullDetail.getPartName());
            detailOffline.setPullQty(orderPartMap.get(pullDetail.getPartNo()).getItemQty());
            detailOffline.setWorkOrderNo(partPull.getWorkCenterCode());
            detailOffline.setWorkCellCode(pullDetail.getWorkCellCode());
            detailOffline.setEquipment(pullDetail.getEquipment());
            detailOffline.setLocation(pullDetail.getPullAddr());
            if (pullDetail.getScanValidationType() != null && pullDetail.getScanValidationType() == 10) {
                detailOffline.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.WHITE.getCode());
            } else {
                detailOffline.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.RED.getCode());
            }
            detailOffline.setCarSeries(pullDetail.getCarSeries());

            ConvertBean.saveOrUpdate(detailOffline,"SPS-OFFLINE");
            partPullOrderDetailOfflineRao.insert(detailOffline);
        }
    }

    //客户零件号
    private MesCustomerPart getMesCustomerPart(String custPartNo, String erpPartNo, String organizeCode) {
        DdlPackBean custPartPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custPartNo, "custPartNo", custPartPackBean);
        DdlPreparedPack.getStringEqualPack(erpPartNo, "erpPartNo", custPartPackBean);
        List<MesCustomerPart> customerPartList = customerPartRao.findByHqlTopWhere(custPartPackBean, 1);
        if (!customerPartList.isEmpty()) {
            return customerPartList.get(0);
        }
        return null;
    }

    //获取车型配置
    private MesCustomerCarModel getMesCustomerCarModel(String custPartNo, String organizeCode, String custCode) {
        DdlPackBean carModelPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(custPartNo, "custPartNo", carModelPackBean);
        DdlPreparedPack.getStringEqualPack(custCode, "custCode", carModelPackBean);
        DdlPreparedPack.getStringEqualPack(organizeCode, "organizeCode", carModelPackBean);
        List<MesCustomerCarModel> carModels = carModelRao.findByHqlTopWhere(carModelPackBean, 1);
        if (carModels.isEmpty()) {
            return null;
        }
        return carModels.get(0);
    }

    //版本号
    private MesProductVersion getMesProductVersion(String partNo, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        ddlPackBean.setOrderByStr(" order by createDatetime desc ");
        List<MesProductVersion> mesProductVersionList = mesProductVersionRDao.findByHqlTopWhere(ddlPackBean, 1);
        if (Objects.isNull(mesProductVersionList) || mesProductVersionList.isEmpty()) {
          return null;
        }
        return mesProductVersionList.get(0);
    }

    private List<MesWorkOrderPart> dismantleBom(String partNo, double qty, String productTime, String bomCode, String organizeCode, String workCellCode) {
        List<MesWorkOrderPart> orderPartList = new ArrayList<>();

        //1.根据bomCode,partNo productTime organizeCode 查询bom清单 bomList
        List<MesBom> bomList = getPlatBom(partNo, productTime, bomCode, organizeCode);
        //2.根据bomList中的subPartNo 查询零件清单 partList
        List<String> partNoList = bomList.stream().map(MesBom::getItemPartNo).distinct().collect(Collectors.toList());

        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", partPackBean);
        List<MesPartSap> partSapList = mesPartSapRao.findByHqlWhere(partPackBean);
        Map<String, MesPartSap> partMap = partSapList.stream().collect(Collectors.toMap(MesPartSap::getPartNo, Function.identity()));


        for (MesBom bom : bomList) {
            MesWorkOrderPart orderPart = new MesWorkOrderPart();
            MesPartSap partSap = partMap.get(bom.getItemPartNo());
            if (partSap == null) {
                return new ArrayList<>();
            }
            List<MesWorkOrderPart> orderParts = orderPartList.stream().filter(item -> item.getPartNo().equals(partSap.getPartNo())).collect(Collectors.toList());
            if (!orderParts.isEmpty()) {
                MesWorkOrderPart tempPart = orderParts.get(0);
                tempPart.setItemQty(tempPart.getItemQty() + bom.getItemQty() * qty);
                continue;
            }
            orderPart.setPartName(partSap.getPartName());
            orderPart.setPartNo(partSap.getPartNo());
            orderPart.setItemQty(bom.getItemQty() * qty);
            orderPart.setItemUnit(partSap.getUnit());
            orderPart.setSourceId(bom.getId());
            orderPart.setShippingPartNo(partSap.getPartNo());
            orderPart.setWorkCellCode(workCellCode);
            orderPartList.add(orderPart);
        }
        //3.遍历bomList
        return orderPartList;
    }

    private List<MesBom> getPlatBom(String partNo, String effectiveTime, String bomCode, String organizeCode) {

        //MES汇报查询BOM是否携带结束查询条件
        MesConfig config = mesConfigService.getCfgValueByCode(organizeCode, MesExtConstWords.MES_REPORT_FIND_BOM_WITH_EFFENDTIME);
        Boolean isWithEffEndTime = (null != config && !org.springframework.util.StringUtils.isEmpty(config.getCfgValue()) && config.getCfgValue().equals(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr())) ? true : false;

        if (bomCode == null) {
            //首先根据虚结构的零件号查询出最新的一条bom信息作为bomCode
            DdlPackBean bomCodePackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, bomCodePackBean);
            DdlPreparedPack.getStringSmallerPack(effectiveTime, MesExtConstWords.EFF_START_TIME, bomCodePackBean);
            if (isWithEffEndTime) DdlPreparedPack.getStringBiggerPack(effectiveTime, MesExtConstWords.EFF_END_TIME, bomCodePackBean);
            DdlPreparedPack.getOrderBy(MesExtConstWords.EFF_START_TIME, CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), bomCodePackBean);
            List<MesBom> bomList = bomRao.findByHqlTopWhere(bomCodePackBean, 1);
            if (bomList == null || bomList.isEmpty()) {
                return new ArrayList<>();
            }
            bomCode = bomList.get(0).getBomCode();
        }
        List<MesBom> bomResultList = new ArrayList<>();
        DdlPackBean bomPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(bomCode, "bomCode", bomPackBean);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.PART_NO, bomPackBean);
        DdlPreparedPack.getStringEqualPack(effectiveTime, MesExtConstWords.EFF_START_TIME, bomPackBean);
        List<MesBom> bomList = bomRao.findByHqlWhere(bomPackBean);

        for (MesBom bom : bomList) {
            if (bom.getPartType() != null && bom.getPartType().equalsIgnoreCase("X")) {
                getPlatBom(bom.getItemPartNo(), effectiveTime, null, organizeCode);
            } else {
                bom.setBomCode(bom.getBomCode().toLowerCase());
                bomResultList.add(bom);
            }

        }

        return bomResultList;
    }

    private List<MesWorkOrderAssembly> getAssemblyList(String workCenterCode, String vehicleNo, List<MesWorkOrderPart> workOrderPartList, MesPartProdGroup mesPartProdGroup) {
        //TM_BAS_WORK_ORDER_ASSEMBLY_SETTING 对应  mes_prod_rule_sort_cfg 排序生产规则
        List<MesWorkOrderAssembly> workOrderAssemblyList = new ArrayList<>();
        List<String> partNoList = workOrderPartList.stream().map(MesWorkOrderPart::getPartNo).collect(Collectors.toList());
        DdlPackBean prodRuleSortPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", prodRuleSortPackBean);
        DdlPreparedPack.getInPackList(partNoList, "assemblyPartNo", prodRuleSortPackBean);
        //查询当前产线的所有排序生产规则--分为当前工单清单的装配件
        List<MesProdRuleSortCfg> sortCfgList = mesProdRuleSortCfgRao.findByHqlWhere(prodRuleSortPackBean);
        List<MesProdRuleSortCfg> sortCfgListFilter = sortCfgList.stream().filter(item -> partNoList.contains(item.getAssemblyPartNo())).collect(Collectors.toList());
        Map<String, List<MesWorkOrderPart>> orderPartMap = workOrderPartList.stream().collect(Collectors.groupingBy(MesWorkOrderPart::getPartNo));
        //遍历排序生产规则
        for (MesProdRuleSortCfg sortCfg : sortCfgListFilter) {
            List<MesWorkOrderPart> orderPartList = orderPartMap.get(sortCfg.getAssemblyPartNo());
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
//            MesWorkOrderPart orderPart = orderPartList.get(0);
            //1.逻辑储量程序sortCfg.isLogicNum
            if ((sortCfg.getIsLogicNum() != null && sortCfg.getIsLogicNum() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) && (sortCfg.getLogicNum() != null && sortCfg.getLogicNum() > 0)) {
                Double total = orderPartList.stream().map(MesWorkOrderPart::getItemQty).reduce(Double::sum).get();
                if (!total.equals(sortCfg.getLogicNum())) {
                    continue;
                }
            }
            //2.当排序规则的车型不为空时，则需要与参数匹配，如果不符合 则不加这个装配项
            if (StringUtils.isNotBlank(sortCfg.getVehicleNo()) && !sortCfg.getVehicleNo().equals(vehicleNo)) {
                continue;
            }
            orderAssembly.setAssemblyPartNo(sortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(sortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(sortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(sortCfg.getWorkCenterCode());
            orderAssembly.setDisplayRule(sortCfg.getDisplayRule());
            orderAssembly.setMatchRule(sortCfg.getMatchRule());
            orderAssembly.setMatchType(sortCfg.getMatchType());
            orderAssembly.setPrintType(sortCfg.getPrintType());
            orderAssembly.setProcessCode(sortCfg.getProcessCode());
            orderAssembly.setDisplaySeq(sortCfg.getDisplaySeq());
            orderAssembly.setDisplayValue(sortCfg.getDisplayValue());
            orderAssembly.setProcessSeq(sortCfg.getProcessSeq());
            orderAssembly.setCraftName(sortCfg.getCraftName());
            orderAssembly.setCraftCode(sortCfg.getCraftCode());
            orderAssembly.setRouteSeq(sortCfg.getRouteSeq());
            orderAssembly.setWorkCellCode(sortCfg.getWorkCellCode());
//            orderAssembly.setOrderPartId(orderPart.getId().toString());
            orderAssembly.setEquipmentCode(sortCfg.getEquipmentCode());
            orderAssembly.setReportType(sortCfg.getReportType());
            if (sortCfg.getReportType() != null && sortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()) {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            } else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(sortCfg.getReportPartNo());
            orderAssembly.setImageFileId(sortCfg.getImageFileId());
            orderAssembly.setCraftCode(sortCfg.getCraftCode());
            orderAssembly.setCraftName(sortCfg.getCraftName());
            orderAssembly.setPartTypeCode(sortCfg.getPartTypeCode());
            orderAssembly.setShowColor(sortCfg.getShowColor());
            orderAssembly.setProductPartNo(sortCfg.getProductPartNo());
            orderAssembly.setPid(sortCfg.getId());
            orderAssembly.setWorkCenterCode(workCenterCode);
            workOrderAssemblyList.add(orderAssembly);
        }

        //车型不为空的逻辑--用于报工
        DdlPackBean reportRuleSortPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", reportRuleSortPackBean);
        DdlPreparedPack.getStringEqualPack(vehicleNo, "vehicleNo", reportRuleSortPackBean);
        DdlPreparedPack.getNotInPack(partNoList, "assemblyPartNo", reportRuleSortPackBean);
        //查询当前产线的所有排序生产规则--分为当前工单清单的装配件
        List<MesProdRuleSortCfg> prodCfgList = mesProdRuleSortCfgRao.findByHqlWhere(reportRuleSortPackBean);
        for (MesProdRuleSortCfg ruleSortCfg : prodCfgList) {
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
            orderAssembly.setAssemblyPartNo(ruleSortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(ruleSortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(ruleSortCfg.getWorkCenterCode());
            orderAssembly.setDisplayRule(ruleSortCfg.getDisplayRule());
            orderAssembly.setMatchRule(ruleSortCfg.getMatchRule());
            orderAssembly.setMatchType(ruleSortCfg.getMatchType());
            orderAssembly.setPrintType(ruleSortCfg.getPrintType());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setWorkCellCode(ruleSortCfg.getWorkCellCode());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setCraftCode(ruleSortCfg.getCraftCode());
            orderAssembly.setRouteSeq(ruleSortCfg.getRouteSeq());
            orderAssembly.setEquipmentCode(ruleSortCfg.getEquipmentCode());
            orderAssembly.setReportType(ruleSortCfg.getReportType());
            if (ruleSortCfg.getReportType() != null && ruleSortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()) {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            } else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(ruleSortCfg.getReportPartNo());
            orderAssembly.setProcessSeq(ruleSortCfg.getProcessSeq());
            orderAssembly.setPid(ruleSortCfg.getId());
            orderAssembly.setImageFileId(ruleSortCfg.getImageFileId());
            orderAssembly.setPartTypeCode(ruleSortCfg.getPartTypeCode());
            orderAssembly.setMaxValue(ruleSortCfg.getMaxValue());
            orderAssembly.setMinValue(ruleSortCfg.getMinValue());
            orderAssembly.setProductPartNo(ruleSortCfg.getProductPartNo());
            workOrderAssemblyList.add(orderAssembly);
        }
        //查询所有共用的零件 且没有零件号，多用于螺钉---查询排序加工规则
        DdlPackBean prodRuleSortPublicPackBean = DdlPackBean.getDdlPackBean(mesPartProdGroup.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", prodRuleSortPublicPackBean);
        DdlPreparedPack.getIsNull("assemblyPartNo", prodRuleSortPublicPackBean);
        List<MesProdRuleSortCfg> ruleSortCfgs = mesProdRuleSortCfgRao.findByHqlWhere(prodRuleSortPublicPackBean);
        for (MesProdRuleSortCfg ruleSortCfg : ruleSortCfgs) {
            MesWorkOrderAssembly orderAssembly = new MesWorkOrderAssembly();
            orderAssembly.setAssemblyPartNo(ruleSortCfg.getAssemblyPartNo());
            orderAssembly.setAssemblyPartName(ruleSortCfg.getAssemblyPartName());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setWorkCenterCode(ruleSortCfg.getWorkCenterCode());
            orderAssembly.setDisplayRule(ruleSortCfg.getDisplayRule());
            orderAssembly.setMatchRule(ruleSortCfg.getMatchRule());
            orderAssembly.setMatchType(ruleSortCfg.getMatchType());
            orderAssembly.setPrintType(ruleSortCfg.getPrintType());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setWorkCellCode(ruleSortCfg.getWorkCellCode());
            orderAssembly.setProcessCode(ruleSortCfg.getProcessCode());
            orderAssembly.setProcessName(ruleSortCfg.getProcessName());
            orderAssembly.setCraftCode(ruleSortCfg.getCraftCode());
            orderAssembly.setRouteSeq(ruleSortCfg.getRouteSeq());
            orderAssembly.setEquipmentCode(ruleSortCfg.getEquipmentCode());
            orderAssembly.setReportType(ruleSortCfg.getReportType());
            if (ruleSortCfg.getReportType() != null && ruleSortCfg.getReportType() == MesExtEnumUtil.MES_REPORT_TYPE.REPORT.getValue()) {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_30.getValue());
            } else {
                orderAssembly.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
            }
            orderAssembly.setReportPartNo(ruleSortCfg.getReportPartNo());
            orderAssembly.setProcessSeq(ruleSortCfg.getProcessSeq());
            orderAssembly.setPid(ruleSortCfg.getId());
            orderAssembly.setImageFileId(ruleSortCfg.getImageFileId());
            orderAssembly.setPartTypeCode(ruleSortCfg.getPartTypeCode());
            orderAssembly.setMaxValue(ruleSortCfg.getMaxValue());
            orderAssembly.setMinValue(ruleSortCfg.getMinValue());
            orderAssembly.setProductPartNo(ruleSortCfg.getProductPartNo());
            workOrderAssemblyList.add(orderAssembly);
        }
        return workOrderAssemblyList;
    }


}
